% Copyright (c) 2014-2015, Leonid Torgovitski
% All rights reserved. 

function [weights] = estimate_Sigma(Ystruct,model,estimate,weights,do_flags)
%ESTIMATE_SIGMA Compute estimates for the exact weighting scheme


display('compute estimates for w')
 
    %%% RESET VALUES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    weights.exact_est_rough                 =   weights.standard*0;  
    weights.exact_est_banded                =   weights.standard*0;   
    weights.exact_est_banded_centered       =   weights.standard*0;
  
    estimate.CVest_rough                  =   estimate.A*0;  
    estimate.CVest_banded                 =   estimate.A*0;  
    estimate.CVest_banded_centered        =   estimate.A*0;
    
    Ystruct.YPrime                          =   0;
    Ystruct.YPrime_centered                 =   0;
    
    
    n   = model.n; 
    p   = model.p; 
    n2  = estimate.n2; 
    
    Ystruct.YPrime          = Ystruct.Y;
    Ystruct.YPrime_centered = Ystruct.YPrime*0;
   
    
    %%% CENTER PANELS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if do_flags.exact_est_center 
       for i=1:p 
           Ystruct.YPrime_centered(:,i) = Ystruct.Y(:,i)-mean(Ystruct.Y(1:n2,i)); 
       end 
    end
      
    %%% ESTIMATE COVARIANCE - NON - CENTERED %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for i=1:n
        for j=1:n
            Z = cov(Ystruct.YPrime(i,:),Ystruct.YPrime(j,:),1);
            estimate.CVest_rough(i,j)=Z(2,1);  
            
            %%% ESTIMATE COVARIANCE - CENTERED %%%%%%%%%%%%%%%%%%%%%%%%%%
            if do_flags.exact_est_center 
                Zcenter = cov(Ystruct.YPrime_centered(i,:),Ystruct.YPrime_centered(j,:),1);
                estimate.CVest_rough_centered(i,j)=Zcenter(2,1); 
            end
        end 
    end
     
        
    %%% ESTIMATE COVARIANCE BY AVERAGING AND BANDING (WE TACITLY ASSUME n1=1
    %%% FOR THE TRAINING PERIOD)
    if do_flags.banded_banded

        h = estimate.h;

        for r=0:h
             navg = (n2-r);

                % NON CENTERED PANELS  
                    S = 0;  
                    S_center=0;
                    for j=1:navg
                        %%% averaging over navg terms for lag-r %%%
                        S = S + estimate.CVest_rough(j,j+r)/navg;

                        if do_flags.exact_est_center 
                            S_center = S_center + estimate.CVest_rough_centered(j,j+r)/navg; 
                        end
                    end

                    %%%% distribute over all positions %%%%%%%%%%%%%
                    for j=1:(n-r)
                        estimate.CVest_banded(j+r,j)  =   S;
                        estimate.CVest_banded(j,j+r)  =   S;

                        if do_flags.exact_est_center
                            estimate.CVest_banded_centered(j+r,j) =   S_center;
                            estimate.CVest_banded_centered(j,j+r) =   S_center;
                        end
                    end   
        end
    end

    %%% SET THE WEIGHTS FOR THE ESTIMATES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for i=(1:(n-1))
        weights.exact_est_rough(i)              = estimate.A(i,:)*estimate.CVest_rough*transpose(estimate.A(i,:));
        weights.exact_est_banded(i)             = estimate.A(i,:)*estimate.CVest_banded*transpose(estimate.A(i,:));
        weights.exact_est_banded_centered(i) 	= estimate.A(i,:)*estimate.CVest_banded_centered*transpose(estimate.A(i,:));
    end 
       
    weights.exact_est_banded            = power(weights.exact_est_banded,-1); 
    weights.exact_est_rough             = power(weights.exact_est_rough,-1); 
    weights.exact_est_banded_centered   = power(weights.exact_est_banded_centered,-1);

      
    
    %%% CHECK FOR NUMERICAL INSTABILITIES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % in case of numerical instability switch back to wstandard 
    
    if ~isempty(find(weights.exact_est_banded<=0)) 
        weights.exact_est_banded = weights.standard;
        warning('\hat{w}-exact-banded is non positive')
    end      

    if ~isempty(find(weights.exact_est_rough<=0)) 
        weights.exact_est_rough = weights.standard;
        warning('\hat{w}-exact is non positive')
    end       

    if ~isempty(find(weights.exact_est_banded_centered<=0))
        weights.exact_est_banded_centered = weights.standard;
        warning('\hat{w}-exact-banded-centered is non positive')
    end          
          
end
 


