% Copyright (c) 2014-2015, Leonid Torgovitski
% All rights reserved.

function [handles] = estimation(handles,do_rand)

fprintf('\n**********\n\n')
display('simulation started')

%% SETUP PART %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


        %%% READ ALL INPUT PARAMETERS FROM THE GUI %%%% 
        %%% CHECK IF INPUT-FORMAT IS CORRECT %%%%%%%%%%
            cont = get(handles.ctrl_n,'String');
            if ~check_isstrint(cont)
                msgbox('n must be an integer');
                return
            end  

            cont = get(handles.ctrl_p,'String');
            if ~check_isstrint(cont)
                msgbox('d must be an integer');
                return
            end 
            
            cont = get(handles.ctrl_u,'String'); 
            if ~check_isstrint(cont)
                msgbox('u must be an integer');
                return
            end
            
            
            cont = get(handles.ctrl_delta2_k,'String'); 
            if ~check_isstrnum(cont)
                msgbox('(delta_k)^2 must be a number');
                return
            end  
            
            cont = get(handles.ctrl_cp_rnd,'String'); 
            if ~check_isstrint(cont)
                msgbox('U must be an integer');
                return
            end  
            
            cont = get(handles.ctrl_phi,'String');
            if ~check_isstrnum(cont)
                msgbox('phi must be a number');
                return
            end   

            cont = get(handles.ctrl_theta,'String');
            if ~check_isstrnum(cont)
                msgbox('theta must be a number');
                return
            end  
              
            cont = get(handles.ctrl_tildesigma2,'String');
            if ~check_isstrnum(cont)
                msgbox('\tilde{sigma}^2 must be a number');
                return
            end  
            
            cont = get(handles.ctrl_gamma,'String');
            if ~check_isstrnum(cont)
                msgbox('gamma must be a number');
                return
            end  
            
            cont = get(handles.ctrl_n2,'String');
            if ~check_isstrint(cont)
                msgbox('n_2 must be an integer');
                return
            end
            
            cont = get(handles.ctrl_h,'String');
            if ~check_isstrint(cont)
                msgbox('h must be an integer');
                return
            end
        
        %%% STORE INPUT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        model.n             =   str2double(get(handles.ctrl_n,'String'));
        model.p             =   str2double(get(handles.ctrl_p,'String'));
         
        model.u             =   str2double(get(handles.ctrl_u,'String'));
        model.s             =   model.u/model.n;
        
        if model.s<1/2 
            model.s = 1-model.s;
        end
        
        model.phi           =   str2double(get(handles.ctrl_phi,'String'));
        model.theta         =   str2double(get(handles.ctrl_theta,'String'));
        model.tildesigma2   =   str2double(get(handles.ctrl_tildesigma2,'String')); 
        
        model.cp_rnd        =   str2double(get(handles.ctrl_cp_rnd,'String'));
        model.delta2_k      =   str2double(get(handles.ctrl_delta2_k,'String'));
       
        weights.gamma       =   str2double(get(handles.ctrl_gamma,'String'));
      
        model.delta2        =   model.delta2_k;
        
        estimate.h          =   str2double(get(handles.ctrl_h,'String'));
        estimate.n2         =   str2double(get(handles.ctrl_n2,'String'));
        
        
        
        %%% CHECK IF INPUT IS REASONABLE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if (model.n<3)
            msgbox('n must be larger than 2');
            return
        end 
        
        if (model.p<2)
            msgbox('d must be larger than 1 - otherwise estimation of w-exact fails');
            return
        end 
        
        if (model.u-model.cp_rnd<1 || model.u+model.cp_rnd>model.n-1)
            msgbox('u-U > 0 and u+U < n required');
            return
        end 
        
        if (model.tildesigma2<=0)
            msgbox('\tilde{sigma}^2 must be positive');
            return
        end 
        
        if (model.delta2_k<=0)
            msgbox('(delta_k)^2 must be positive');
            return
        end 
                 
        if (weights.gamma<0 || weights.gamma>1/2)
            msgbox('gamma must be between 0 and 0.5');
            return
        end
        
        if (estimate.n2<2 || estimate.n2>model.u-model.cp_rnd)
            msgbox('n_2 must be larger than n_1=1 and not larger than u-U');
            return
        end
        
        if (estimate.h<0 || estimate.h>estimate.n2 || estimate.h>model.n)
            msgbox('h must be non-negative and not larger than n_2');
            return
        end
         
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
        %%% BASIC WEIGHTS (SQUARED) %%%
        misc.hvec           =   1:(model.n-1);
        weights.standard    =   power((misc.hvec/model.n).*(1-misc.hvec/model.n),-1);
        weights.weighted    =   power((misc.hvec/model.n).*(1-misc.hvec/model.n),-(2*weights.gamma));
        weights.exact_est_rough                 =   weights.standard*0; 
        weights.exact_est_banded              =   weights.standard*0;          
        weights.exact_est_banded_centered     =   weights.standard*0; 
    
        n                   =   model.n;
        phi                 =   model.phi; 
        model.C             =   model.tildesigma2*(1+model.theta^2); 
        model.sigma2        =   model.tildesigma2*(1+model.phi^2+model.theta^2+model.phi^2*model.theta^2);
        model.vfunct2       =   model.C*((1+phi^2+2*phi+2*phi/n)./weights.standard-2*phi/n)/model.sigma2;    
     

        %%% PREPARE THE MATRIX A FOR estimate_Sigma.m %%%
        estimate.A          =   zeros(n, n); 
        for i=1:n
            for j=1:n
                if j>i
                    estimate.A(i,j) = (-i/n)/sqrt(n);
                else
                    estimate.A(i,j) = (1-i/n)/sqrt(n);
                end
            end
        end 
        
        model.p_arr         =   model.p; 
        model.phi_arr       =   [model.phi];
        model.length_phi    =   length(model.phi_arr);
        model.length_parr   =   length(model.p_arr);

        do_flags.rand = do_rand;

        do_flags.simple             =   get(handles.chk_simple,'Value');
        do_flags.standard           =   get(handles.chk_standard,'Value');
        do_flags.weighted           =   get(handles.chk_weighted,'Value');
        do_flags.exact              =   get(handles.chk_exact,'Value');
                
        do_flags.exact_est          =   get(handles.chk_exact_rough,'Value');
        do_flags.banded_banded    =   get(handles.chk_exact_banded,'Value');
        do_flags.exact_est_center   =   get(handles.chk_exact_banded_centered,'Value'); 
         
        do_flags.rescaling          =   get(handles.chk_rescale,'Value');
        do_flags.showweights        =   get(handles.chk_showweights,'Value');
        
        output = NaN;
       
        Ystruct.Etas = NaN;
        
        if do_flags.rescaling ==1
            demonstration.rescaling = n;
        else
            demonstration.rescaling = 1;
        end
        

%% SIMULATION PART %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         
tic;
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     
    Ystruct     =   generate_data(Ystruct,model,do_flags);
    Ystruct.Y 	=   add_change(Ystruct.E,model);                 
    
    n   =   model.n;        
    weights.exact   =   power(model.vfunct2,-1);   
    
    if do_flags.exact_est
        [weights]   =    estimate_Sigma(Ystruct,model,estimate,weights,do_flags);
    end
    
    %%% COMPUTE AND ESTIMATE t(i)'s %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Ystruct.Ycentered   =   Ystruct.Y - kron(ones(n,1),mean(Ystruct.Y,1));
    Ystruct.Ycusum      =   cumsum(Ystruct.Ycentered(1:(n-1),:),1);
    Ystruct.Ysum        =   sum(Ystruct.Ycusum.^2,2); 
    tYsum               =   transpose(Ystruct.Ysum);   
    
    Ystruct.Ywsimple    =   Ystruct.Ysum;
    Ystruct.Ywweighted  =   tYsum.*(weights.weighted);
    Ystruct.Ywexact     =   tYsum.*(weights.exact); 
    Ystruct.Ywstandard 	=   tYsum.*(weights.standard); 
    
    Ystruct.Ywexact_est_rough             =   tYsum.*(weights.exact_est_rough); 
    Ystruct.Ywexact_est_banded            =   tYsum.*(weights.exact_est_banded);
    Ystruct.Ywexact_est_banded_centered   =   tYsum.*(weights.exact_est_banded_centered); 

    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
toc; 
        


%% DEMONSTRATION PART %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	%%% CALCULATE lim R AND B(gamma) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    s = model.s;
    
    %%% CALCULATE B(gamma) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    gamma=weights.gamma;
    
    if gamma<1/2
        bfuncgamma = (4*gamma^2 +6*gamma^(3/2)-3*gamma^(1/2)-1)/(8*gamma^2 + 8*gamma^(3/2) -4*gamma^(1/2)-2*gamma-1);
    else
        bfuncgamma = 1/sqrt(2);
    end
    
    %%% SET LABELS FOR OUTPUT AND CALCULATE lim R %%%%%%%%%%%%%%%%%%%%%%%%
    set(handles.txt_ratio,'string',['<HTML>ratio r=',num2str(model.sigma2/(model.delta2*n)),'<HTML>']);
    set(handles.txt_bgamma_label,'string',['<HTML>B(',num2str(gamma),')=',num2str(bfuncgamma),'<HTML>']);
    set(handles.txt_s,'string',['<HTML>Rescaled location s=',num2str(s),' where s=u/n for u larger than n/2 and s= (n-u)/n for u smaller than n/2.</HTML>']);
    
    if model.theta==0 && model.phi == 0
        
        if s<bfuncgamma && s>0.5
            
                Rin = abs((model.s*(1-model.s)*((weights.gamma-1)/(2*weights.gamma-1)-model.s)/(model.s-1/2)));
                set(handles.txt_s_smaller_bgamma,'string',['<HTML>Since s&isin;(0.5,B(',num2str(gamma),')) we have lim &real;(',num2str(s),',',num2str(gamma),')=',num2str(Rin),'</HTML>'])
        
        elseif s>bfuncgamma && gamma==0.25
            
                Routfunc = ((s-1)^2*(s*(s+1)+(s/2-1)*(1-s)^(1/2)-1))/(s*(s-1)+(s/2)*(1-s)^(1/2));
                set(handles.txt_s_smaller_bgamma,'string',['<HTML>We have lim &real;(',num2str(s),',',num2str(gamma),')=',num2str(Routfunc),'</HTML>'])
        
        elseif s == 0.5
            
            set(handles.txt_s_smaller_bgamma,'string',['<HTML>We have lim &real;(',num2str(s),',',num2str(gamma),')=&#8734;</HTML>'])
        
        else
            
            set(handles.txt_s_smaller_bgamma,'string','<HTML>lim &real;(s,&#947;) is defined only for &#952;=&phi;=0. Furthermore, we need either s&isin;(1/2,B(&#947;)) or &#947;= 1/4</HTML>');
        
        end
    else
        
        set(handles.txt_s_smaller_bgamma,'string','<HTML>lim &real;(s,&#947;) is defined only for &#952;=&phi;=0. Furthermore, we need either s&isin;(1/2,B(&#947;)) or &#947;= 1/4</HTML>');
    
    end
    
    %%% PLOTTING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    col_arr = distinguishable_colors(10);
    
    
    ix.simple = 1;
    ix.weighted = 2;
    ix.standard = 3;
    ix.exact = 4;
    
    ix.exact_est = 5;  
    ix.exact_banded = 6;
    ix.exact_banded_centered =7; 
    
    axes(handles.axes_max);
      
    if do_flags.showweights 
        
        plotsarr = (1:7);
    
        %figure
        plot(1);
        
        hold on
            ix_plt = [false,false,false,false,false,false];

            if do_flags.weighted 
                plotsarr(ix.weighted)=plot_wrapper_weights(weights.weighted,demonstration.rescaling,col_arr(ix.weighted,:),['weighted gamma=',num2str(weights.gamma)]);
                ix_plt(ix.weighted) = true;
            end
            
            if do_flags.standard 
                plotsarr(ix.standard)=plot_wrapper_weights(weights.standard,demonstration.rescaling,col_arr(ix.standard,:),'standard');
                ix_plt(ix.standard) = true;
            end        
            
            if do_flags.exact 
                plotsarr(ix.exact)=plot_wrapper_weights(weights.exact,demonstration.rescaling,col_arr(ix.exact,:),'exact');
                ix_plt(ix.exact) = true;
            end
            
            if do_flags.exact_est 
                plotsarr(ix.exact_est)=plot_wrapper_weights(weights.exact_est_rough,demonstration.rescaling,col_arr(ix.exact_est,:),'exact-est');
                ix_plt(ix.exact_est) = true;
            end
           
            if do_flags.banded_banded 
                plotsarr(ix.exact_banded)=plot_wrapper_weights(weights.exact_est_banded,demonstration.rescaling,col_arr(ix.exact_banded,:),'\hat{w}-ex.-band.');
                ix_plt(ix.exact_banded) = true;
            end
        
            if do_flags.exact_est_center
                plotsarr(ix.exact_banded_centered)=plot_wrapper_weights(weights.exact_est_banded_centered,demonstration.rescaling,col_arr(ix.exact_banded_centered,:),'ex.-est.-band-cent.');
                ix_plt(ix.exact_banded_centered) = true;
            end

        hold off 
        xlabel('i');
    else
        
        plotsarr = (1:7);
    
        plot(1);
        
        hold on
            ix_plt = [false,false,false,false,false,false,false,false,false];
            
            if do_flags.simple 
                plotsarr(ix.simple)     =   plot_wrapper(Ystruct.Ywsimple,demonstration.rescaling,col_arr(ix.simple,:),'simple');
                ix_plt(ix.simple)       =   true;
            end

            if do_flags.weighted
                plotsarr(ix.weighted)   =   plot_wrapper(Ystruct.Ywweighted,demonstration.rescaling,col_arr(ix.weighted,:),['weighted gamma=',num2str(weights.gamma)]);
                ix_plt(ix.weighted)     =   true;
            end
            
            if do_flags.standard
                plotsarr(ix.standard)   =   plot_wrapper(Ystruct.Ywstandard,demonstration.rescaling,col_arr(ix.standard,:),'standard');
                ix_plt(ix.standard)     =   true;
            end  

            if do_flags.exact
                plotsarr(ix.exact)      =   plot_wrapper(Ystruct.Ywexact,demonstration.rescaling,col_arr(ix.exact,:),'exact');
                ix_plt(ix.exact)        =   true;
            end

            if do_flags.exact_est
                plotsarr(ix.exact_est)  =   plot_wrapper(Ystruct.Ywexact_est_rough,demonstration.rescaling,col_arr(ix.exact_est,:),'ex.-est.');
                ix_plt(ix.exact_est)    =   true;
            end
            
            if do_flags.banded_banded
                plotsarr(ix.exact_banded)  =   plot_wrapper(Ystruct.Ywexact_est_banded,demonstration.rescaling,col_arr(ix.exact_banded,:),'ex.-est.-banded');
                ix_plt(ix.exact_banded)    =   true;
            end
               
            
           if do_flags.exact_est_center
                plotsarr(ix.exact_banded_centered)   =   plot_wrapper(Ystruct.Ywexact_est_banded_centered,demonstration.rescaling,col_arr(ix.exact_banded_centered,:),'ex.-est.-band.-cent.');
                ix_plt(ix.exact_banded_centered)     =   true;
           end
  

        hold off
 
        xlim([0,n/demonstration.rescaling]);
        xlabel('i');
    end
    
    
            
    %%% CALCULATE CRITICAL FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        critfunc.basic                          =   (1:(n-1))*0; 
        critfunc.basic(1:model.u)               =   ((1:model.u)/n).*(1-model.u/n);
        critfunc.basic((model.u+1):(n-1)) =   (model.u/n).*(1-((model.u+1):(n-1))/n);
        
        critfunc.simple                         =   (model.vfunct2*(model.sigma2/model.delta2)/n + critfunc.basic.^2);
        critfunc.weighted                       =   critfunc.simple.*weights.weighted;
        critfunc.standard                       =   critfunc.simple.*weights.standard;
        critfunc.exact                          =   critfunc.simple.*weights.exact;

    %%% PLOTTING CRITICAL FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        axes(handles.axes_critfunc);
         
        plot(1);

            hold on
            plotsarr = (1:4);
            ix_plt = [false,false,false,false];
              
            if do_flags.simple
                plotsarr(ix.simple) =   plot_wrapper(critfunc.simple,demonstration.rescaling,col_arr(ix.simple,:),'simple');
                ix_plt(ix.simple)   =   true;
            end

            if do_flags.weighted
                plotsarr(ix.weighted)   =   plot_wrapper(critfunc.weighted,demonstration.rescaling,col_arr(ix.weighted,:),['weighted gamma=',num2str(weights.gamma)]);
                ix_plt(ix.weighted)     =   true;
            end

            if do_flags.standard
                plotsarr(ix.standard)   =   plot_wrapper(critfunc.standard,demonstration.rescaling,col_arr(ix.standard,:),'standard');
                ix_plt(ix.standard)     =   true;
            end
            
            if do_flags.exact
                plotsarr(ix.exact)      =   plot_wrapper(critfunc.exact,demonstration.rescaling,col_arr(ix.exact,:),'exact');
                ix_plt(ix.exact)        =   true;
            end
            
        hold off 
            
        xlim([0,n/demonstration.rescaling]);
        xlabel('i');
        
        
        
        %%% WORKAROUND - ONLY FOR SHOWING THE LEGEND (OTHERWISE MATLAB
        %%% CHANGES ALWAYS THE SIZE OF THE PLOTS) 
        axes(handles.axes_workaround); 
        plotsarr_wa = (1:7);
        ix_plt_wa = [false,false,false,false,false,false,false];
        plot(1);

        hold on

       
        if do_flags.simple
            plotsarr_wa(ix.simple)            =   plot(1,'Color',col_arr(ix.simple,:),'DisplayName','w-simple');
            ix_plt_wa(ix.simple)              =   true;
        end
  
        
        if do_flags.weighted
            plotsarr_wa(ix.weighted)            =   plot(1,'Color',col_arr(ix.weighted,:),'DisplayName',['w-weighted gamma=',num2str(weights.gamma)]);
            ix_plt_wa(ix.weighted)              =   true;
        end
  

        if do_flags.standard
            plotsarr_wa(ix.standard)            =   plot(1,1,'Color',col_arr(ix.standard,:),'DisplayName','w-standard');
            ix_plt_wa(ix.standard)                 =   true;
        end
        

        if do_flags.exact
            plotsarr_wa(ix.exact)               =   plot(1,1,'Color',col_arr(ix.exact,:),'DisplayName','w-exact');
            ix_plt_wa(ix.exact)                    =   true;
        end

        if do_flags.exact_est
            plotsarr_wa(ix.exact_est)           =   plot(1,7,'Color',col_arr(ix.exact_est,:),'DisplayName','\hat{w}-ex.');
            ix_plt_wa(ix.exact_est)                =   true;
        end

        if do_flags.banded_banded
            plotsarr_wa(ix.exact_banded)        =   plot(1,13,'Color',col_arr(ix.exact_banded,:),'DisplayName','\hat{w}-ex.-band.');
            ix_plt_wa(ix.exact_banded)             =   true;
        end

        if do_flags.exact_est_center
            plotsarr_wa(ix.exact_banded_centered)=   plot(1,2,'Color',col_arr(ix.exact_banded_centered,:),'DisplayName','\hat{w}-ex.-band-cent.');
            ix_plt_wa(ix.exact_banded_centered)     =   true;
        end

        hold 'off'

        legend(plotsarr_wa(ix_plt_wa),'Location','southeastoutside');
        legend('boxoff');
  

display('simulation finished')
fprintf('\n**********\n')
end



function [plt] = plot_wrapper(v,rescaling,mycol,capt)
    v       =   (v - min(v))/(max(v)-min(v));
    hvec    =   (1:length(v))/rescaling;
    plt     =   plot(hvec,v,'Color',mycol,'DisplayName',capt);
    [argval, argmax] = max(v); 
    line([argmax/rescaling argmax/rescaling], get(gca,'ylim'),'Color',mycol); 
end


function [plt] = plot_wrapper_weights(v,rescaling,mycol,capt)
    v       =   (v - min(v))/(max(v)-min(v));
    hvec    =   (1:length(v))/rescaling;
    plt     =   plot(hvec, v,'Color', mycol,'DisplayName',capt);
end


function [v] = check_isstrnum(cont)
    dg_cont = find(isstrprop(cont, 'digit')==0);
    if length(dg_cont)==0 || (length(dg_cont)==1 && strcmp(cont(1),'-')) || (length(dg_cont)==1 && strcmp(cont(dg_cont(1)),'.') && dg_cont(1)>1) || (length(dg_cont)==2 && strcmp(cont(1),'-') && strcmp(cont(dg_cont(2)),'.'))
        v=1;
    else 
        v=0;
    end   
end

function [v] = check_isstrint(cont)
    
    dg_cont = find(isstrprop(cont, 'digit')==0);
    if length(dg_cont)>0 
        v=0;
    else
        v=1; 
    end
end


